

/* 2007 Microchip Technology Inc.

Microchip licenses this software to you solely for use with Microchip products.
This software is owned by Microchip and its licensors, and is protected under applicable copyright laws. All rights reserved.
SOFTWARE IS PROVIDED AS IS. MICROCHIP EXPRESSLY DISCLAIMS ANY WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
To the fullest extent allowed by law, Microchip and its licensors liability shall not exceed the amount of fees, if any, that you have paid directly to Microchip to use this software.
MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE TERMS




~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

   Filename:               udice_main.c
   Date:                   August 15, 2007
   File Version:           1.0
   Compile using:        MPLAB IDE v7.60 and CCS C compiler

   Author:                 Marc McComb
   Company:                Microchip Technology, Inc.

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

   Files required:         16f57.h

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

   Purpose:

	This code is for the Microdice application.

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

   Program Description:
 
   The following code makes use of the three general-purpose Input/Output 
   Ports as well as the 8-bit Timer0 on the Baseline PIC16F57 MCU to 
   generate a dice algorithm. If one of the pushbuttons connected to 
   either RA0 or RA1 is pressed, the current value in a continuously 
   running random number generator is reserved and used to generate one of 
   six possible outputs to the appropriate dice. PORTB and PORTC are each 
   connected to an LED display to show the determined side of the dice. 
   CCS compiler specific functions are used to implement all delays.
   
   **Remember some functions and preprocessor directives are specific
   to the CCS compiler. If another compiler is used these functions and
   preprocessor directives will generate errors or warnings.
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

*/




#include <16f57.h>


#fuses RC,NOWDT,NOPROTECT
#use delay(clock=100000) //configures the device to implement a 100000Hz clock
#define SLEEP_Time	10	//defines how long until PIC MCU goes to sleep

unsigned char PORTA, PORTB, PORTC,TMR0; //declares 4 8-bit registers

//The #locate directive is used to specify register address in data memory
#locate	PORTA = 0X05
#locate	PORTB = 0X06
#locate PORTC = 0X07
#locate TMR0 = 0X01



//variable declaration
int16 LFSR = 1;
int1 Output_bit;
unsigned char 	DIE_OUT;
unsigned char Animate;
int Tumble;
unsigned char DIE_Flag;
unsigned char Time_2_SLEEP;


//Define Dice output to light appropriate LEDs

unsigned char DIE[6] = {0x08,0x41,0x49,0x55,0x5D,0x77};


//#bit directive assigns a specific bit in a declared register to 
//a 1-bit location
#bit	ROLL1 = PORTA.1
#bit	ROLL2 = PORTA.0
#bit	BUZZER = PORTA.2

#bit	TOF = TMR0.7


#bit LFSR_16 = LFSR.15
#bit LFSR_5 = LFSR.4
#bit LFSR_4 = LFSR.3
#bit LFSR_3 = LFSR.2
#bit LFSR_2 = LFSR.1
#bit LFSR_1 = LFSR.0


//Function Prototypes
void Initialize (void);
void Get_Inputs (void);
void Output_DIE(void);
void randnum_generator(void);
void Check_button(void);
void Check_for_SLEEP(void);
void dummy(void);




//The main() containing the software control loop

void main(void)

{	
	

	Initialize();
	
 while(1)
 {
	 delay_ms(100); 	//CCS compiler specific function, implements
	 			//a delay in ms as determined by argument
	 					
	 randnum_generator(); //generate a random number
	 Check_button(); //Check for a button press
	 Check_for_SLEEP(); //Check if it is time to goto sleep


	}
}


//This function initializes all variables and Special Function Registers

void Initialize (void)
{
	//In order to initialize our PORT directions
	//in-line Assembly is required in order to 
	//access the TRIS(PORT direction)register.
	
	//Set the PORT directions as follows:
	//	PORTB and PORTC pins are connected to the 
	//	left and right dice LEDS repectively. 
	//	Therefore, set both these registers as output.
	
	//	PORTA is used to take inputs from the ROLL1 and 
	//	ROLL2 pushbuttons on RA2 and RA1 respectively.
	//	RA3 will be used as the output to our buzzer.
	//	Therefore, set RA1/RA2 as inputs and RA3 as output
	
	//Initialize the PORTS to a known state (i.e. 0)
		PORTA = 0;
		PORTB = 0;
		PORTC = 0;
	
	#asm	//start a block of assembly code
		
		
		movlw	0x03	//Setup PORTA for RA1 and RA2 as inputs
						//and the rest as outputs by moving 0x03
						//into the working register (clearing a pin's
						//direction to '0' makes it an output, setting
						//a pin's direction to '1' makes it an input
						
						//then
						
		tris	PORTA	//issuing the tris command moves the 
						//contents of the working register into
						//the PORTA direction register
						
		movlw	0X00	//PORTB and PORTC pins are used to output to the
						//LEDS. Therefore, clear all bits in the PORTB and
						//PORTC direction registers
						
						
		tris	PORTB
		tris	PORTC
		
		movlw	0x07
		
		OPTION
		
	#endasm	//end a block of Assembly code
	

	PORTB = 0XFF;
	PORTC = 0XFF;
	TMR0 = 0;
	Time_2_SLEEP = SLEEP_Time;

}
	
		
			
//This is the random number generator based off of a Linear Feedback
//Shift Register Algorithm

void randnum_generator(void)
{
	
		Output_bit = LFSR_16; //reserve the most significant bit
		LFSR <<=1;	//Shift register contents Left by one
			
		//XOR the LFSR register with the characteristic polynomial
		//X^16 + X^5 + X^3 + X^2 + 1 and store result in the Least
		//Significant Bit in LFSR		
		  LFSR_1 = Output_bit ^ LFSR_16 ^LFSR_5 ^LFSR_3^LFSR_2;
		 
		 
		 //Check for an all zero condition in the LFSR register
		  if((LFSR & 0X00FF) == 0) LFSR = 1; //if the LFSR is zero reload with 1
		   
		   
}


void Check_button(void)
{
	
	//Check if the dice 1 pushbutton is pressed
	
	if(ROLL1 == 0)
	{
		delay_ms(2); //Delay for 2ms to debounce the switch
		
		//while the ROLL1 button is pressed increment the current
		//LFSR value
		Animate = 6;
		while(ROLL1 == 0) 
			{
				
			++LFSR;
			delay_ms(50);
			
			//animate variable is used to call values from the 
			//DIE array and display a dice side onto the LEDs
			--Animate;
			if(Animate > 5)Animate = 5;
			PORTB = DIE[Animate];
			
			//Toggle the RA3 bit to generate a clicking sound on
			//the buzzer
			BUZZER ^= 1;			
			
		   }
		   
		 //Once the pushbutton is released this while loop generates
		 //a slowing clicking sound and dice display
		 Tumble = 10;
		 while(Tumble <= 40)
		 {
		
			delay_ms(Tumble);
			--Animate; 
			if(Animate > 5)Animate = 5;
			PORTB = DIE[Tumble%5];
			BUZZER ^= 1;
			++Tumble;	
		 }
		
		
		//The DIE_Flag is used to determine which LED display will 
		//light in the Output_DIE function
		DIE_Flag = 1;
		
		//Reload the Time_2_SLEEP variable with the defined SLEEP_Time value
		//since a pushbutton was 
		Time_2_SLEEP = SLEEP_Time;
		
		Output_DIE(); //outputs the result to the DIE LEDs
			
	}
	

	if(ROLL2 == 0)
	{
		delay_ms(2); //Delay for 2ms to debounce the switch
		
		//while the ROLL2 button is pressed increment the current
		//LFSR value
		Animate = 6;
		while(ROLL2 == 0) 
			{
				
			++LFSR;
			delay_ms(50);
			--Animate;
			if(Animate > 5)Animate = 5;
			PORTC = DIE[Animate];
			BUZZER ^= 1;			
			
		   }
		 Tumble = 10;
		 while(Tumble <= 40)
		 {
			 ++LFSR;
			delay_ms(Tumble);
			--Animate;
			if(Animate > 5)Animate = 5;
			PORTC = DIE[Animate];
			BUZZER ^= 1;
			++Tumble;	
		 }
		
		DIE_Flag = 0;
		Time_2_SLEEP = SLEEP_Time;
		Output_DIE(); //output the result to the DIE LEDs
			
	}
		
	
}


//This function decrements the Time_2_SLEEP variable if a pushbutton has not been
//pressed. If the Time_2_SLEEP variable reaches zero, the LED displays are turned
//off and the SLEEP() is executed placing the PIC MCU into low-powered SLEEP mode
void Check_for_SLEEP(void)
{
	if(Time_2_SLEEP==0) 
	 {	 
		 
		 PORTB = 0;
	       PORTC = 0;
		 SLEEP();
	}

	//if Time_2_SLEEP times out go to SLEEP
	

	
	else
		{	while(TOF);
			if(!TMR0) 
			{
				--Time_2_SLEEP;

			}
	
		}
}


//This is the dummy() used when debugging to act as a tag copying the 
//contents of the DIE_OUT register to a .txt file. 
//Note: to use the Debugger Stimulus tool in this way will require some
//modification to the code as main never calls the Output_DIE function
void dummy(void)
{
}



//This function mods the current value of LFSR by 6 and uses the result
//to call one of the dice display patterns from the DIE[] array

void Output_DIE(void)
{	   
	dummy();
 //Moding a number by a value will return a remainder no greater
 //than that value  
	DIE_OUT = (LFSR % 6);
	if(DIE_Flag) PORTB = DIE[DIE_OUT];
	if(!DIE_Flag) PORTC = DIE[DIE_OUT];
		   	
}
		   
			   
		   
		